/**********************************************************
	Main.c
	Boot mode and flash self programming for MCD05 v.1.1

						2015.7.21	M.Ustumi@ARCDEVICE

************************************************************/

#include"MCD05_bt.h"
#include"ESPIx_bt.h"

#define CM_MASTER_ID		0xFF
#define CM_SET_ID 			4
#define CM_GET_ID	 		5

#define RTSP_CONNECT		20
#define RTSP_ERASE			21
#define RTSP_WRITE			22
#define RTSP_READ			23
#define RTSP_RUN			24
#define RTSP_TEST			25

#define PROGRAMING_MODE		30
#define WRITER_MODE			31


BOOL EraseCode( void )
{
	WORD wAddr; 
	WORD w;

	if(!ESPI1_Read( &wAddr, 1 )) return( FALSE );
	ESPI1_ReadDisable();

	FLASH_Erase_32dw( wAddr );

	w = 'a';
	ESPI1_WriteEnable();
	if(!ESPI1_Write( &w, 1 ) ) return( FALSE );
	ESPI1_WriteDisable();
	
	return( TRUE );
}

BOOL WriteCode( void )
{
	WORD wAddr,wSize,wWriteNum; 
	WORD w, wCode[128];
	
	if(!ESPI1_Read( &wAddr, 1 )) return( FALSE );
	if(!ESPI1_Read( &wSize, 1 )) return( FALSE );

	ESPI1_ReadDisable();

	wWriteNum = wSize / 8;

	w = 'a';
	ESPI1_WriteEnable();
	if(!ESPI1_Write( &w, 1 )) return( FALSE );
	ESPI1_WriteDisable();

	ESPI1_ReadEnable();
	if(!ESPI1_Read( wCode, wSize )) return( FALSE );
	ESPI1_ReadDisable();
	
	for(w=0; w<wWriteNum; w++){
		FLASH_Write_4dw( wAddr, &wCode[w*8]  );
		wAddr += 8;
	}

	w = 'a';
	ESPI1_WriteEnable();
	if(!ESPI1_Write( &w, 1 ) ) return( FALSE );
	ESPI1_WriteDisable();

	return( TRUE );
}

BOOL ReadCode( void )
{
	WORD wAddr,wSize,wReadNum;
	WORD w, wCode[128];

	if(!ESPI1_Read( &wAddr, 1 )) return( FALSE );
	if(!ESPI1_Read( &wSize, 1 )) return( FALSE );
	ESPI1_ReadDisable();

	wReadNum = wSize / 2;
	
	for(w=0;w<wReadNum;w++ ){
		FLASH_Read_dw( wAddr, &wCode[w*2] );
		wAddr+=2;
	}	

	ESPI1_WriteEnable();
	if(!ESPI1_Write( wCode, wSize )) return( FALSE );
	
	w = 'a';
	if(!ESPI1_Write( &w, 1 )) return( FALSE );
	ESPI1_WriteDisable();

	return( TRUE );
}

BOOL ProgramingMode( void )
{
	WORD wCommand;
	BOOL ret;

	LED_ON();
	ret = TRUE;
	while(ret){
		ESPI1_ReadEnable();
		if(!ESPI1_Read( &wCommand, 1 )) return( FALSE );

		switch(wCommand){
		case RTSP_ERASE: ret = EraseCode(); break;
		case RTSP_WRITE: ret = WriteCode(); break;
		case RTSP_READ: ret = ReadCode();  break;
		case RTSP_RUN: return( TRUE );
		case RTSP_TEST: UT_Blink(1); break;
		
		default: while(1) UT_Blink(4); break; //ret = FALSE; break; 
		}
		LED=~LED;
	}
	LED_ON();
	return( ret ); 
}

WORD _EEDATA(2) EEP_wID=1;	//vOݎ̏ ID

BYTE GetID( void )
{
	WORD w;

	EEP_Read( 
		__builtin_tblpage(&EEP_wID),
		__builtin_tbloffset(&EEP_wID),
	 	&w, 1 );
	return( (BYTE)w );
}

void SetID( BYTE bID )
{
	WORD w;

	w = bID;
	EEP_Write( 
		__builtin_tblpage(&EEP_wID),
		__builtin_tbloffset(&EEP_wID),
	 	&w, 1 );
}


int main(void)
{
	WORD w;
	BYTE bID, bCommand;
	BYTE *pb;

	UT_SetTimeout( 5.0f );	// 5 sec

	ESPI1_Open();
	ESPI1_IDReceiveEnable();

	TRIS_LED = 0; // output
	LED_ON();
	bID = GetID();
	UT_Wait_ms(2000);

	if(!ESPI1_DataReady() ){
		ESPI1_Close();
		asm("GOTO 0x00100");
	}
	ESPI1_Read( &w, 1 );
	ESPI1_IDReceiveDisable();
	pb = (BYTE*)&w;	
	if( (pb[0] != bID )&&(pb[0]!=CM_MASTER_ID) ){
		ESPI1_Close();
		asm("GOTO 0x00100");
	}
	bCommand = pb[1];
	switch(bCommand){
	case CM_SET_ID:
		ESPI1_Read( &w, 1 );
		bID = (BYTE)w;
		SetID( bID );
		ESPI1_WriteEnable();
		ESPI1_Write( &w, 1 );
		ESPI1_WriteDisable();
		break;
	case CM_GET_ID:
		w = bID;
		ESPI1_WriteEnable();
		ESPI1_Write( &w, 1 );
		ESPI1_WriteDisable();
		break;
	case PROGRAMING_MODE:
		UT_Blink(2);
		w = bCommand;
		ESPI1_WriteEnable();
		ESPI1_Write( &w, 1 );
		ESPI1_WriteDisable();
		
		if(ProgramingMode()){
			ESPI1_Close();
			asm("GOTO 0x00100");
		}
		while(1) UT_Blink(1); // Error

		break;
	default:
		w = 0;
		ESPI1_WriteEnable();
		ESPI1_Write( &w, 1 );
		ESPI1_WriteDisable();
		break; 
	}
	while(1)  UT_Blink(4);	
}


